#include "local.h"
#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include "son.h"
#include "em.typ"

double atof();
Rpoly *add_rpoly();

readpoly(poly_index,lev_index,filename,box,cir,file,use_level,level_offset,lossless,cnvt)
int *poly_index; /* RETURN: polygon index of polygon read		*/
int *lev_index;  /* RETURN: level index of polygon read			*/
char *filename;	/* Name of file being read */
Box_info *box;
Cir *cir;
FILE *file;
int lossless;	/* if True, use lossess as metallization */
int use_level;  /* if TRUE then use level found in file, else offset by */
		/* level_offset						*/
int level_offset;/* If use_level=TRUE, then add level_offset to 	*/
		 /* level defined in the file				*/
Rtyp cnvt;	/* conversion factor for values read in 		*/
{
	char Str[256];
	int ilev,mode,icond;
        int id,xmin,ymin,xmax,ymax;
	int npts;
	char *Pos;
	Rtyp x,y;
	Rpoly *poly;

	/*Load the circuit polygons*/
	if ( sFgets(Str,255,file)==NULL ) { 
		fprintf(stderr,"File \"%s\", line %d: Missing first data in polygon.\n",filename,Fgets_line_num() ); 
		return(-1); 
	}
	Pos = Skip_blnks(Str);

	ilev = atoi(Pos);
	/* if paste file set the level */
	if ( use_level ) {
		ilev += level_offset;
		if ( ilev < 0 ) ilev = 0;
		if ( ilev >  box->Num_levels ) ilev = box->Num_levels;
	}

	npts = atoi(Pos=Next_field(Pos));
	if (npts<3 ) { 
		fprintf(stderr,"File \"%s\", line %d: Polygon has less than three points.\n",filename,Fgets_line_num() );  
		return(-1); 
	}
	icond = atoi(Pos=Next_field(Pos));  /*polygon conductivity*/
	if ( lossless ) icond = -1;
	if (icond==0 && *Pos!='0') icond = -1;  /*default*/
	if (icond < -1 || icond >= box->num_met) { 
		fprintf(stderr,"File \"%s\", line %d: Polygon conductivity '%d' out of range.\n",filename,Fgets_line_num() ,icond);  
		return(-1); 
	}
	mode = STAIRCASE_FILL;
	if ( *(Pos=Next_field(Pos)) == 'T' || *Pos == 't' ) mode = TRI_FILL;
	if ( *Pos == 'C' || *Pos == 'c' ) mode = COR_FILL;
	if ( *Pos == 'B' || *Pos == 'b' ) mode = TRI_FILL | COR_FILL;

        id = atoi(Pos=Next_field(Pos));
        if ( strlen(Pos=Next_field(Pos)) ) xmin = atoi(Pos);
        else xmin = -999;
        if ( strlen(Pos=Next_field(Pos)) ) ymin = atoi(Pos);
        else ymin = -999;
        if ( strlen(Pos=Next_field(Pos)) ) xmax = atoi(Pos);
        else xmax = -999;
        if ( strlen(Pos=Next_field(Pos)) ) ymax = atoi(Pos);
        else ymax = -999;

	if (ilev<0 || ilev> box->Num_levels ) { 
		fprintf(stderr,"File \"%s\", line %d: Level number in polygon is out of range.\n",filename,Fgets_line_num() );  
		return(-1); 
	}
	poly = add_rpoly(cir->plev0+ilev,NULL,0);
	poly->mode = mode;  poly->icond = icond;
        poly->id = id;
        if (xmin != -999) poly->xmin = xmin;
        if (ymin != -999) poly->ymin = ymin;
        if (xmax != -999) poly->xmax = xmax;
        if (ymax != -999) poly->ymax = ymax;
	while ( (npts--)>0 ) { 
		if ( sFgets(Str,255,file)==NULL ) { 
			fprintf(stderr,"File \"%s\", line %d: Missing vertice, polygon.\n",filename,Fgets_line_num() );  
			return(-1); 
		}
		Pos = Skip_blnks(Str);
		if (strncmp(Pos,"END",3)==0) {
			fprintf(stderr,"File \"%s\", line %d: Encounted END before all vertices for polygon were defined\n",
				filename,Fgets_line_num() );  
			return(-1);
		}
		if ( *Pos == '\0' ) {
			fprintf(stderr,"File \"%s\", line %d: X coordindate not defined polygon.\n",
				filename,Fgets_line_num() );  
			return(-1);
		}
		x = cnvt * atof(Pos) *  box->Lcnv;
		Pos = Next_field(Pos);
		if ( *Pos == '\0' ) {
			fprintf(stderr,"File \"%s\", line %d: Y coordindate not defined polygon.\n",
				filename,Fgets_line_num() );  
			return(-1);
		}
		y = cnvt * atof(Pos) *  box->Lcnv;
		add_rpoly_pt(poly,x,y);
	}
	if ( sFgets(Str,255,file)==NULL ) { 
		fprintf(stderr,"File \"%s\", line %d: Missing END at end of polygon.\n",filename,Fgets_line_num() );  
		return(-2); 
	}

	if ( strncmp(Str, "END", 3) != 0 ) {
		fprintf(stderr,"File \"%s\", line %d: Missing END at end of polygon.\n",filename,Fgets_line_num() );  
		return(-1); 
	}

	/* Polygon just added is last one on the polygon list */
	*poly_index = (cir->plev0+ilev)->n-1;	
	*lev_index = ilev;
	return(0);
}
